@@ -85,7 +85,7 @@
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
 /*
-/*	Scheduler enhancements:
+/*	Preemptive scheduler enhancements:
 /*	Patrik Rak
 /*	Modra 6
 /*	155 00, Prague, Czech Republic
@@ -334,6 +334,7 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
     int     dsn_notify = 0;
     char   *dsn_orcpt = 0;
     int     n;
+    int     have_log_client_attr = 0;
 
     /*
      * Initialize. No early returns or we have a memory leak.
@@ -653,18 +654,27 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	     * client information. To support old queue files we accept both
 	     * names for the purpose of logging; the new name overrides the
 	     * old one.
+	     * 
+	     * XXX Do not use the "legacy" client_name etc. attribute values for
+	     * initializing the logging attributes, when this file already
+	     * contains the "modern" log_client_name etc. logging attributes.
+	     * Otherwise, logging attributes that are not present in the
+	     * queue file would be set with information from the real client.
 	     */
 	    else if (strcmp(name, MAIL_ATTR_ACT_CLIENT_NAME) == 0) {
-		if (message->client_name == 0)
+		if (have_log_client_attr == 0 && message->client_name == 0)
 		    message->client_name = mystrdup(value);
 	    } else if (strcmp(name, MAIL_ATTR_ACT_CLIENT_ADDR) == 0) {
-		if (message->client_addr == 0)
+		if (have_log_client_attr == 0 && message->client_addr == 0)
 		    message->client_addr = mystrdup(value);
+	    } else if (strcmp(name, MAIL_ATTR_ACT_CLIENT_PORT) == 0) {
+		if (have_log_client_attr == 0 && message->client_port == 0)
+		    message->client_port = mystrdup(value);
 	    } else if (strcmp(name, MAIL_ATTR_ACT_PROTO_NAME) == 0) {
-		if (message->client_proto == 0)
+		if (have_log_client_attr == 0 && message->client_proto == 0)
 		    message->client_proto = mystrdup(value);
 	    } else if (strcmp(name, MAIL_ATTR_ACT_HELO_NAME) == 0) {
-		if (message->client_helo == 0)
+		if (have_log_client_attr == 0 && message->client_helo == 0)
 		    message->client_helo = mystrdup(value);
 	    }
 	    /* Original client attributes. */
@@ -672,22 +682,27 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 		if (message->client_name != 0)
 		    myfree(message->client_name);
 		message->client_name = mystrdup(value);
+		have_log_client_attr = 1;
 	    } else if (strcmp(name, MAIL_ATTR_LOG_CLIENT_ADDR) == 0) {
 		if (message->client_addr != 0)
 		    myfree(message->client_addr);
 		message->client_addr = mystrdup(value);
+		have_log_client_attr = 1;
 	    } else if (strcmp(name, MAIL_ATTR_LOG_CLIENT_PORT) == 0) {
 		if (message->client_port != 0)
 		    myfree(message->client_port);
 		message->client_port = mystrdup(value);
+		have_log_client_attr = 1;
 	    } else if (strcmp(name, MAIL_ATTR_LOG_PROTO_NAME) == 0) {
 		if (message->client_proto != 0)
 		    myfree(message->client_proto);
 		message->client_proto = mystrdup(value);
+		have_log_client_attr = 1;
 	    } else if (strcmp(name, MAIL_ATTR_LOG_HELO_NAME) == 0) {
 		if (message->client_helo != 0)
 		    myfree(message->client_helo);
 		message->client_helo = mystrdup(value);
+		have_log_client_attr = 1;
 	    } else if (strcmp(name, MAIL_ATTR_SASL_METHOD) == 0) {
 		if (message->sasl_method == 0)
 		    message->sasl_method = mystrdup(value);
@@ -1069,22 +1084,6 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 			  "5.1.3 null recipient address");
 	}
 
-	/*
-	 * Bounce recipient addresses that start with `-'. External commands
-	 * may misinterpret such addresses as command-line options.
-	 * 
-	 * In theory I could say people should always carefully set up their
-	 * master.cf pipe mailer entries with `--' before the first
-	 * non-option argument, but mistakes will happen regardless.
-	 * 
-	 * Therefore the protection is put in place here, in the queue manager,
-	 * where it cannot be bypassed.
-	 */
-	if (var_allow_min_user == 0 && recipient->address[0] == '-') {
-	    QMGR_REDIRECT(&reply, MAIL_SERVICE_ERROR,
-			  "5.1.3 bad address syntax");
-	}
-
 	/*
 	 * Discard mail to the local double bounce address here, so this
 	 * system can run without a local delivery agent. They'd still have
